@@ -1,3 +1,4 @@
+import asyncio
 import logging
 import os
 import platform
@@ -6,7 +7,6 @@ import sys
 import typing
 
 import click
-from h11._connection import DEFAULT_MAX_INCOMPLETE_EVENT_SIZE
 
 import uvicorn
 from uvicorn.config import (
@@ -78,9 +78,6 @@ def print_version(ctx: click.Context, param: click.Parameter, value: bool) -> No
 @click.option(
     "--fd", type=int, default=None, help="Bind to socket from this file descriptor."
 )
-@click.option(
-    "--debug", is_flag=True, default=False, help="Enable debug mode.", hidden=True
-)
 @click.option("--reload", is_flag=True, default=False, help="Enable auto-reload.")
 @click.option(
     "--reload-dir",
@@ -337,7 +334,7 @@ def print_version(ctx: click.Context, param: click.Parameter, value: bool) -> No
 )
 @click.option(
     "--app-dir",
-    default=".",
+    default="",
     show_default=True,
     help="Look for APP in the specified directory, by adding this to the PYTHONPATH."
     " Defaults to the current working directory.",
@@ -346,7 +343,7 @@ def print_version(ctx: click.Context, param: click.Parameter, value: bool) -> No
     "--h11-max-incomplete-event-size",
     "h11_max_incomplete_event_size",
     type=int,
-    default=DEFAULT_MAX_INCOMPLETE_EVENT_SIZE,
+    default=None,
     help="For h11, the maximum number of bytes to buffer of an incomplete event.",
 )
 @click.option(
@@ -371,7 +368,6 @@ def main(
     ws_per_message_deflate: bool,
     lifespan: LifespanType,
     interface: InterfaceType,
-    debug: bool,
     reload: bool,
     reload_dirs: typing.List[str],
     reload_includes: typing.List[str],
@@ -401,7 +397,7 @@ def main(
     headers: typing.List[str],
     use_colors: bool,
     app_dir: str,
-    h11_max_incomplete_event_size: int,
+    h11_max_incomplete_event_size: typing.Optional[int],
     factory: bool,
 ) -> None:
     run(
@@ -423,11 +419,10 @@ def main(
         log_level=log_level,
         access_log=access_log,
         interface=interface,
-        debug=debug,
         reload=reload,
-        reload_dirs=reload_dirs if reload_dirs else None,
-        reload_includes=reload_includes if reload_includes else None,
-        reload_excludes=reload_excludes if reload_excludes else None,
+        reload_dirs=reload_dirs or None,
+        reload_includes=reload_includes or None,
+        reload_excludes=reload_excludes or None,
         reload_delay=reload_delay,
         workers=workers,
         proxy_headers=proxy_headers,
@@ -455,55 +450,54 @@ def main(
 
 
 def run(
-    app: typing.Union["ASGIApplication", str],
+    app: typing.Union["ASGIApplication", typing.Callable, str],
     *,
     host: str = "127.0.0.1",
     port: int = 8000,
     uds: typing.Optional[str] = None,
     fd: typing.Optional[int] = None,
     loop: LoopSetupType = "auto",
-    http: HTTPProtocolType = "auto",
-    ws: WSProtocolType = "auto",
+    http: typing.Union[typing.Type[asyncio.Protocol], HTTPProtocolType] = "auto",
+    ws: typing.Union[typing.Type[asyncio.Protocol], WSProtocolType] = "auto",
     ws_max_size: int = 16777216,
-    ws_ping_interval: float = 20.0,
-    ws_ping_timeout: float = 20.0,
+    ws_ping_interval: typing.Optional[float] = 20.0,
+    ws_ping_timeout: typing.Optional[float] = 20.0,
     ws_per_message_deflate: bool = True,
     lifespan: LifespanType = "auto",
     interface: InterfaceType = "auto",
-    debug: bool = False,
     reload: bool = False,
-    reload_dirs: typing.Optional[typing.List[str]] = None,
-    reload_includes: typing.Optional[typing.List[str]] = None,
-    reload_excludes: typing.Optional[typing.List[str]] = None,
+    reload_dirs: typing.Optional[typing.Union[typing.List[str], str]] = None,
+    reload_includes: typing.Optional[typing.Union[typing.List[str], str]] = None,
+    reload_excludes: typing.Optional[typing.Union[typing.List[str], str]] = None,
     reload_delay: float = 0.25,
     workers: typing.Optional[int] = None,
-    env_file: typing.Optional[str] = None,
+    env_file: typing.Optional[typing.Union[str, os.PathLike]] = None,
     log_config: typing.Optional[
         typing.Union[typing.Dict[str, typing.Any], str]
     ] = LOGGING_CONFIG,
-    log_level: typing.Optional[str] = None,
+    log_level: typing.Optional[typing.Union[str, int]] = None,
     access_log: bool = True,
     proxy_headers: bool = True,
     server_header: bool = True,
     date_header: bool = True,
-    forwarded_allow_ips: typing.Optional[str] = None,
+    forwarded_allow_ips: typing.Optional[typing.Union[typing.List[str], str]] = None,
     root_path: str = "",
     limit_concurrency: typing.Optional[int] = None,
     backlog: int = 2048,
     limit_max_requests: typing.Optional[int] = None,
     timeout_keep_alive: int = 5,
     ssl_keyfile: typing.Optional[str] = None,
-    ssl_certfile: typing.Optional[str] = None,
+    ssl_certfile: typing.Optional[typing.Union[str, os.PathLike]] = None,
     ssl_keyfile_password: typing.Optional[str] = None,
-    ssl_version: int = int(SSL_PROTOCOL_VERSION),
-    ssl_cert_reqs: int = int(ssl.CERT_NONE),
+    ssl_version: int = SSL_PROTOCOL_VERSION,
+    ssl_cert_reqs: int = ssl.CERT_NONE,
     ssl_ca_certs: typing.Optional[str] = None,
     ssl_ciphers: str = "TLSv1",
     headers: typing.Optional[typing.List[typing.Tuple[str, str]]] = None,
     use_colors: typing.Optional[bool] = None,
     app_dir: typing.Optional[str] = None,
     factory: bool = False,
-    h11_max_incomplete_event_size: int = DEFAULT_MAX_INCOMPLETE_EVENT_SIZE,
+    h11_max_incomplete_event_size: typing.Optional[int] = None,
 ) -> None:
     if app_dir is not None:
         sys.path.insert(0, app_dir)
@@ -523,7 +517,6 @@ def run(
         ws_per_message_deflate=ws_per_message_deflate,
         lifespan=lifespan,
         interface=interface,
-        debug=debug,
         reload=reload,
         reload_dirs=reload_dirs,
         reload_includes=reload_includes,
@@ -573,7 +566,7 @@ def run(
         Multiprocess(config, target=server.run, sockets=[sock]).run()
     else:
         server.run()
-    if config.uds:
+    if config.uds and os.path.exists(config.uds):
         os.remove(config.uds)  # pragma: py-win32
 
     if not server.started and not config.should_reload and config.workers == 1:
